@@ -18,14 +18,38 @@
 
 import os
 import statvfs
-import os.path
-
-lib_dir    = os.path.abspath (os.path.dirname (__file__))
-prefix_dir = os.path.abspath (os.path.join (lib_dir, "..", "..", ".."))
-data_dir   = os.path.join (prefix_dir, "share", "serpentine")
-locale_dir = os.path.join (prefix_dir, "share", "locale")
+import sys
+import tempfile
 
 from gettext import gettext as _
+from os import path
+
+
+class ApplicationLocations:
+    def __init__(self, root, appname):
+        self.root = path.abspath(root)
+        pyver = "python%d.%d" % sys.version_info[0:2]
+        self.lib = path.join(root, "lib", pyver, "site-packages")
+        self.data = path.join(root, "share", appname)
+        self.bin = path.join(root, "bin")
+        self.locale = path.join(root, "share", "locale")
+
+    def get_data_file(self, filename):
+        return path.join(self.data, filename)
+    
+    def get_lib_file(self, filename):
+        return path.join(self.lib, filename)
+
+    def get_bin_file(self, filename):
+        return path.join(self.bin, filename)
+
+    def get_locale_file(self, filename):
+        return path.join(self.locale, filename)
+
+    def get_root_file(self, filename):
+        return path.join(self.root, filename)
+
+
 
 class SerpentineError (StandardError): pass
 
@@ -72,7 +96,6 @@ def __plural (value, strings):
     else:
         return strings[1]
 
-import tempfile, os
 
 class SafeFileWrite:
     """This class enables the user to safely write the contents to a file and
@@ -82,7 +105,7 @@ class SafeFileWrite:
     
     def __init__ (self, filename):
         self.filename = filename
-        basedir = os.path.dirname (filename)
+        basedir = path.dirname (filename)
         # must be in the same directory so that renaming works
         fd, self.tmp_filename = tempfile.mkstemp (dir = basedir)
         os.close (fd)
@@ -133,7 +156,7 @@ def validate_music_list (music_list, preferences):
     if (size_avail - size_needed) < 0:
         preferences.pool.clear ()
     
-    size_avail = size_avail = s[statvfs.F_BAVAIL] * long(s[statvfs.F_BSIZE])
+    size_avail = s[statvfs.F_BAVAIL] * long(s[statvfs.F_BSIZE])
     if (size_avail - size_needed) < 0:
         raise SerpentineCacheError (SerpentineCacheError.NO_SPACE, _("Remove " 
                                     "some music tracks or make sure your "     
