@@ -19,22 +19,19 @@
 import gtk
 import os
 import os.path
-import tempfile
 import gobject
 
 # Local imports
 import operations
 import gaw
-import constants
 import gtkutil
 
 from components import Component
-from operations import MapProxy, OperationListener, OperationsQueue
-from operations import CallableOperation
-from mastering import AudioMastering, GtkMusicList, MusicListGateway
-from mastering import ErrorTrapper
+from operations import MapProxy, OperationListener
+from mastering import AudioMastering
 
-from serpentine.common import *
+from serpentine.common import SerpentineNotSupportedError, validate_music_list
+from serpentine.common import SerpentineCacheError
 from gettext import gettext as _
 
 class GladeComponent (Component):
@@ -139,7 +136,7 @@ class SavePlaylistComponent (GladeComponent):
         hbox = gtk.HBox (spacing = 6)
         hbox.show ()
         
-        lbl = gtk.Label(_("Save playlist in format")+":")
+        lbl = gtk.Label(_("Save playlist in format:"))
         lbl.show ()
         hbox.pack_start (lbl, False, False)
         
@@ -168,13 +165,7 @@ class SavePlaylistComponent (GladeComponent):
     def on_finished (self, evt):
         win = self.parent
             
-        if evt.id == operations.SUCCESSFUL:
-            gtkutil.dialog_warn (
-                _("Playlist Saved"),
-                _("Playlist was saved successfully."),
-                parent = win
-            )
-        else:
+        if evt.id != operations.SUCCESSFUL:
             gtkutil.dialog_error (
                 _("Playlist Not Saved"),
                 _("There was an error while saving the playlist."),
@@ -232,12 +223,54 @@ class SavePlaylistComponent (GladeComponent):
                 break
 
             except SerpentineNotSupportedError:
-                gtkutil.dialog_error (
-                    _("Unsupported Format"),
-                    _("The playlist format you used (by the file extension) is "
-                    "currently not supported."),
-                    parent = win
-                )
+                # In this case the user supplied a wrong extension
+                # let's ask for him to choose one
+                
+                if extension is None:
+                    # convert to strings
+                    items = map(lambda row: row[0], self.store)
+                    # First row is useless
+                    del items[0]
+                    
+                    indexes, response = gtkutil.choice_dialog(
+                        _("Select one playlist format"),
+                        _("Serpentine will open any of these formats so the "
+                          "one you choose only matters if you are going to "
+                          "open with other applications."),
+                        one_text_item = _("Do you want to save as the %s "
+                                          "format?"),
+                        min_select = 1,
+                        max_select = 1,
+                        parent = win,
+                        items = items,
+                        ok_button = gtk.STOCK_SAVE,
+                    )
+                    if len(indexes) != 0:
+                        index, = indexes
+                        # Since we deleted the first row from the items then
+                        # the index have an offset of 1
+                        index += 1
+                        row = self.store[index]
+                        extension = row[1]
+                        
+                        # Save the file
+                        oper = app.savePlaylist.save (filename, extension)
+                        oper.listeners.append (self)
+                        oper.start ()
+                        
+                        # Select the option in the list store
+                        self.combo.set_active(index)
+                        break
+                        
+                        
+                else:
+                    gtkutil.dialog_error (
+                        _("Unsupported Format"),
+                        _("The playlist format you used (by the file extension) is "
+                        "currently not supported."),
+                        parent = win
+                    )
+                    
             
         self.file_dlg.unselect_all()
         self.file_dlg.hide()
@@ -348,14 +381,14 @@ class SerpentineWindow (gtk.Window, OperationListener, operations.Operation, Com
         Component.__init__ (self, application)
             
         self.__application = application
-        self.__masterer = AudioMastering ()
+        self.__masterer = AudioMastering (application)
         # Variable related to this being an Operation
         self.__running = False
         self.connect ("show", self.__on_show)
         # We listen for GtkMusicList events
         self.music_list_widget.listeners.append (self)
-
-        glade_file = os.path.join (constants.data_dir, "serpentine.glade")
+        
+        glade_file = application.locations.get_data_file("serpentine.glade")
         g = gtk.glade.XML (glade_file, "main_window_container")
         
         # Send glade to setup subcomponents
@@ -477,7 +510,7 @@ class SerpentineWindow (gtk.Window, OperationListener, operations.Operation, Com
         a.set_name ("Serpentine")
         a.set_version (self.__application.preferences.version)
         a.set_website ("http://s1x.homelinux.net/projects/serpentine")
-        a.set_copyright ("2004-2005 Tiago Cogumbreiro")
+        a.set_copyright ("2004-2006 Tiago Cogumbreiro")
         a.set_transient_for (self)
         a.run ()
         a.hide()
