@@ -18,26 +18,22 @@
 # Authors: Tiago Cogumbreiro <cogumbreiro@users.sf.net>
 
 import gtk
-import gtk.glade
 import gobject
-import os.path
-import urllib
-import gnomevfs
-import gettext
 import sys
+import weakref
 
-from xml.dom import minidom
+from gtk import glade
+from os import path
 from types import IntType, TupleType
-from urlparse import urlparse, urlunparse
 from gettext import gettext as _
+from gettext import ngettext as N_
 
 # Local modules
 import operations
 import audio
 import xspf
-import constants
 import gtkutil
-import serpentine.common
+import urlutil
 
 from gtkutil import DictStore
 from operations import OperationsQueue
@@ -52,10 +48,10 @@ class ErrorTrapper (operations.Operation, operations.OperationListener):
     def __init__ (self, parent = None):
         operations.Operation.__init__ (self)
         self.__errors = []
-        self.__parent = parent
+        self._parent = parent
     
     errors = property (lambda self: self.__errors)
-    parent = property (lambda self: self.__parent)
+    parent = property (lambda self: self._parent)
     
     def on_finished (self, event):
         if event.id == operations.ERROR:
@@ -67,26 +63,34 @@ class ErrorTrapper (operations.Operation, operations.OperationListener):
             for l in self.listeners:
                 l.on_finished (e)
             return
-                
-        elif len (self.errors) > 1:
-            title = _("Unsupported file types")
-        else:
-            title = _("Unsupported file type")
-            
+
         filenames = []
         for e in self.errors:
-            filenames.append (gnomevfs.URI(e.hints['location']).short_name)
+            filenames.append (urlutil.basename(e.hints['location']))
         del self.__errors
         
-        if len (filenames) == 1:
-            msg = _("The following files were not added:") + "\n"
-        else:
-            msg = _("The following files were not added:") + "\n"
-        
-        msg += " "
-        msg += ", ".join (filenames)
+        title = N_(
+            "Unsupported file type",
+            "Unsupported file types",
+            len(filenames)
+        )
 
-        gtkutil.dialog_error (title, msg, parent = self.parent)
+        msg = N_(
+            "The following file was not added:",
+            "The following files were not added:",
+            len(filenames)
+        )
+
+        gtkutil.list_dialog(
+            title,
+            _("If you're having problems opening certain files make sure you "
+              "have the GStreamer plugins needed to decode them."),
+            list_title=msg,
+            parent=self.parent,
+            items=filenames,
+            stock = gtk.STOCK_DIALOG_ERROR,
+            buttons =(gtk.STOCK_CLOSE, gtk.RESPONSE_OK),
+        )
         
         e = operations.FinishedEvent (self, operations.SUCCESSFUL)
         for l in self.listeners:
@@ -97,23 +101,48 @@ class AddFile (audio.AudioMetadataListener, operations.Operation):
     
     running = property (lambda self: False)
 
-    def __init__ (self, music_list, hints, insert = None):
+    def __init__ (self, music_list, hints, insert=None, app=None):
         operations.Operation.__init__ (self)
         self.hints = hints
         self.music_list = music_list
         self.insert = insert
+        # We normalize the paths by default
+        hints["location"] = urlutil.normalize(hints["location"])
+        self.app = app
     
     def start (self):
-        oper = audio.gvfsAudioMetadata (self.hints['location'])
+        if self.app.preferences.useGnomeVfs:
+            oper = audio.get_metadata(audio.GVFS_SRC, self.hints["location"])
+            
+        else:
+            url = urlutil.UrlParse(self.hints["location"])
+            if not url.is_local:
+                self._send_finished_error(
+                    operations.ERROR,
+                    error=StandardError(self.hints["location"])
+                )
+                return
+                
+            filename = url.path
+            oper = audio.get_metadata("filesrc", filename)
+
         oper.listeners.append (self)
-        oper.start()
+        try:
+            oper.start()
+        except audio.GstPlayingFailledError:
+            self._send_finished_event(
+                operations.ERROR,
+                error=StandardError(self.hints["location"])
+            )
     
     def on_metadata (self, event, metadata):
+        title = urlutil.basename(self.hints['location'])
+        title = path.splitext(title)[0]
         
         row = {
             "location": self.hints['location'],
             "cache_location": "",
-            "title": gnomevfs.URI(self.hints['location'][:-4]).short_name or _("Unknown"),
+            "title": title or _("Unknown"),
             "artist": _("Unknown Artist"),
             "duration": int(metadata['duration']),
         }
@@ -223,15 +252,16 @@ class GtkMusicList (MusicList):
     Takes care of the data source. Supports events and listeners.
     """
     SPEC = (
-            # URI is used in converter
-            {"name": "location", "type": gobject.TYPE_STRING},
-            # filename is used in recorder
-            {"name": "cache_location", "type": gobject.TYPE_STRING},
-            # Remaining items are for the list
-            {"name": "duration", "type": gobject.TYPE_INT},
-            {"name": "title", "type": gobject.TYPE_STRING},
-            {"name": "artist", "type": gobject.TYPE_STRING},
-            {"name": "time", "type": gobject.TYPE_STRING})
+        # URI is used in converter
+        {"name": "location", "type": gobject.TYPE_STRING},
+        # filename is used in recorder
+        {"name": "cache_location", "type": gobject.TYPE_STRING},
+        # Remaining items are for the list
+        {"name": "duration", "type": gobject.TYPE_INT},
+        {"name": "title", "type": gobject.TYPE_STRING},
+        {"name": "artist", "type": gobject.TYPE_STRING},
+        {"name": "time", "type": gobject.TYPE_STRING}
+    )
             
     def __init__ (self):
         operations.Listenable.__init__ (self)
@@ -381,6 +411,9 @@ class HintsFilter (object):
         assert isinstance (value, HintsFilter)
         return self.priority - value.priority
 
+def normalize_hints(hints):
+    hints["location"] = urlutil.normalize(hints["location"])
+
 class MusicListGateway:
     """This class wraps the MusicList interface in a friendlier one with a
     method `add_files` easier to use then the `insert` method which expects
@@ -401,9 +434,10 @@ class MusicListGateway:
         def prepare_add_file (self, gateway, add_file):
             """Method called before add_file object is added to queue"""
     
-    def __init__ (self):
+    def __init__ (self, app):
         # Filters
         self.__filters = []
+        self._app = weakref.ref(app)
     
     music_list = None
     
@@ -415,8 +449,8 @@ class MusicListGateway:
         return None
     
     def add_files (self, filenames):
-        hints = [{"location":file} for file in filenames]
-        return self.add_hints (hints)
+        to_hint = lambda filename: {"location": urlutil.normalize(filename)}
+        return self.add_hints(map(to_hint, filenames))
 
     def add_hints (self, hints_list, insert = None):
         assert insert is None or isinstance (insert, IntType)
@@ -431,7 +465,10 @@ class MusicListGateway:
         i = 0
         for h in hints_list:
             pls = self.__filter_location (h["location"])
+            
             if pls is not None and len (pls) > 0:
+                # normalize the returning elements
+                map(normalize_hints, pls)
                 # We add this to the queue so it is
                 # processed before the next file on the list
                 queue.append (self.add_hints(pls, insert))
@@ -441,7 +478,7 @@ class MusicListGateway:
             if insert != None:
                 ins += i
             
-            a = AddFile (self.music_list, h, ins)
+            a = AddFile (self.music_list, h, ins, self._app())
             handler.prepare_add_file (self, a)
             
             queue.append (a)
@@ -459,6 +496,7 @@ class MusicListGateway:
     def remove_hints_filter (self, location_filter):
         self.__filters.remove (location_filter)
 
+
 class AudioMastering (gtk.VBox, operations.Listenable):
     SIZE_21 = 0
     SIZE_74 = 1
@@ -467,7 +505,7 @@ class AudioMastering (gtk.VBox, operations.Listenable):
     
     class MusicListGateway (MusicListGateway):
         def __init__ (self, parent):
-            MusicListGateway.__init__ (self)
+            MusicListGateway.__init__ (self, parent._application())
             self.parent = parent
         
         def music_list (self):
@@ -502,8 +540,9 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         ('text/plain', 0, 2),
         ('STRING', 0, 3),
     ]
-    def __init__ (self):
+    def __init__ (self, application):
         gtk.VBox.__init__ (self)
+        self._application = weakref.ref(application)
         
         operations.Listenable.__init__ (self)
         self.__disc_size = 74 * 60
@@ -514,8 +553,9 @@ class AudioMastering (gtk.VBox, operations.Listenable):
         self.__gateway = AudioMastering.MusicListGateway (self)
         
         gtk.VBox.__init__ (self)
-        g = gtk.glade.XML (os.path.join (constants.data_dir, "serpentine.glade"),
-                           "audio_container")
+        filename = application.locations.get_data_file("serpentine.glade")
+        g = glade.XML (filename, "audio_container")
+        
         self.add (g.get_widget ("audio_container"))
         
         self.__setup_track_list (g)
@@ -658,9 +698,10 @@ class AudioMastering (gtk.VBox, operations.Listenable):
             
         for line in data.split("\n"):
             line = line.strip()
-            if len (line) < 1:
+            if len (line) < 1 or line == "\x00":
                 continue
                 
+            assert "\x00" not in line, "Malformed DnD string: %s" % line
             hint = {'location': line}
             hints_list.append (hint)
         self.music_list_gateway.add_hints (hints_list, insert).start ()
@@ -686,6 +727,19 @@ class AudioMastering (gtk.VBox, operations.Listenable):
                 
             hig_duration += hig_secs
         return hig_duration
+    # TODO: implement get_media_duration
+
+    def get_preferences(self):
+        return self._application().preferences
+        
+    def get_media_duration(self):
+        total = self.source.total_duration
+        num_tracks = len(self.source)
+        
+        if num_tracks > 0 and self.get_preferences().useGap:
+            total += (num_tracks - 1) * 2
+        
+        return total 
     
     def update_disc_usage (self):
         if not self.update:
@@ -742,7 +796,6 @@ class AudioMastering (gtk.VBox, operations.Listenable):
 
     
 if __name__ == '__main__':
-    import sys, os
     win = gtk.Window()
     win.connect ("delete-event", gtk.main_quit)
     w = AudioMastering ()
