@@ -19,7 +19,6 @@
 
 """Uses totem.plparser to try and load playlists."""
 
-import gnomevfs
 from totem import plparser
 
 from serpentine.mastering import HintsFilter
@@ -35,21 +34,11 @@ class PlparserFilter (HintsFilter):
         hints_list.append(hints)
 
     def filter_location (self, location):
-        
-        try:
-            mime = gnomevfs.get_mime_type (location)
-        except RuntimeError:
-            return
-            
-        if mime == "audio/x-mpegurl" or mime == "audio/x-scpls":
-            hints_list = []
-            p = plparser.Parser()
-            p.connect("entry", self.__on_pl_entry, hints_list)
-            p.parse(location, False)
-
+        hints_list = []
+        p = plparser.Parser()
+        p.connect("entry", self.__on_pl_entry, hints_list)
+        if p.parse(location, False) == plparser.PARSER_RESULT_SUCCESS:
             return hints_list
-            
-        return
     
 def create_plugin (serpentine_object):
     serpentine_object.register_playlist_file_pattern ("PLS Audio Playlist", "*.pls")
